@@ -39,7 +39,8 @@ scPalette <- c('#E41A1C', '#377EB8', '#4DAF4A', '#FFCF00', '#aa47b9', '#e67c14',
 .ligerOptions <- list(
     ligerBaseSize = 10,
     ligerVerbose = TRUE,
-    ligerDotSize = 1
+    ligerDotSize = 1,
+    ligerChunkSize = 2e4
 )
 
 .onLoad <- function(libname, pkgname) {
