@@ -1,14 +1,13 @@
 #' SNN Graph Based Community Detection
 #' @description
-#' After quantile normalization, users can additionally run the Leiden or
+#' After aligning cell factor loadings, users can additionally run the Leiden or
 #' Louvain algorithm for community detection, which is widely used in
 #' single-cell analysis and excels at merging small clusters into broad cell
 #' classes.
 #'
-#' While using quantile normalized factor loadings (result from
-#' \code{\link{quantileNorm}}) is recommended, this function looks for
-#' unnormalized factor loadings (result from \code{\link{runIntegration}}) when
-#' the former is not available.
+#' While using aligned factor loadings (result from \code{\link{alignFactors}})
+#' is recommended, this function looks for unaligned factor loadings (raw result
+#' from \code{\link{runIntegration}}) when the former is not available.
 #' @param object A \linkS4class{liger} object. Should have valid factorization
 #' result available.
 #' @param nNeighbors Integer, the maximum number of nearest neighbors to
@@ -56,50 +55,55 @@
 #' head(pbmcPlot$leiden_cluster)
 #' pbmcPlot <- runCluster(pbmcPlot, method = "louvain")
 #' head(pbmcPlot$louvain_cluster)
-runCluster <- function(
-        object,
-        resolution = 1.0,
-        nNeighbors = 20,
-        prune = 1 / 15,
-        eps = 0.1,
-        nRandomStarts = 10,
-        nIterations = 5,
-        method = c("leiden", "louvain"),
-        useRaw = NULL,
-        useDims = NULL,
-        groupSingletons = TRUE,
-        saveSNN = FALSE,
-        clusterName = paste0(method, "_cluster"),
-        seed = 1,
-        verbose = getOption("ligerVerbose", TRUE)
-) {
+runCluster <- function(object,
+                       resolution = 1.0,
+                       nNeighbors = 20,
+                       prune = 1 / 15,
+                       eps = 0.1,
+                       nRandomStarts = 10,
+                       nIterations = 5,
+                       method = c("leiden", "louvain"),
+                       useRaw = NULL,
+                       useDims = NULL,
+                       groupSingletons = TRUE,
+                       saveSNN = FALSE,
+                       clusterName = paste0(method, "_cluster"),
+                       seed = 1,
+                       verbose = getOption("ligerVerbose", TRUE)) {
     method <- match.arg(method)
-    object <- switch(method,
+    object <- switch(
+        method,
         leiden = recordCommand(object, dependencies = c("RANN", "leidenAlg")),
         louvain = recordCommand(object, dependencies = c("RANN"))
     )
     Hsearch <- searchH(object, useRaw)
     H <- Hsearch$H
     useRaw <- Hsearch$useRaw
-    type <- ifelse(useRaw, "unnormalized", "quantile normalized")
+    type <- ifelse(useRaw, "unaligned", "aligned")
 
-    if (!is.null(useDims)) H <- H[, useDims, drop = FALSE]
+    if (!is.null(useDims))
+        H <- H[, useDims, drop = FALSE]
 
     if (isTRUE(verbose))
         cli::cli_process_start("{method} clustering on {type} cell factor loadings")
     knn <- RANN::nn2(H, k = nNeighbors, eps = eps)
     snn <- ComputeSNN(knn$nn.idx, prune = prune)
-    if (!is.null(seed)) set.seed(seed)
+    if (!is.null(seed))
+        set.seed(seed)
     if (method == "leiden") {
         snnSummary <- summary(snn)
         edgelist <- as.vector(t(snnSummary[, c(2, 1)])) - 1
         edgelist_length <- length(edgelist)
-        edge_weights <- snnSummary[,3]
+        edge_weights <- snnSummary[, 3]
         clusts <- leidenAlg::find_partition_with_rep_rcpp(
-            edgelist = edgelist, edgelist_length = edgelist_length,
-            num_vertices = nrow(H), direction = FALSE,
-            edge_weights = edge_weights, resolution = resolution,
-            niter = nIterations, nrep = nRandomStarts
+            edgelist = edgelist,
+            edgelist_length = edgelist_length,
+            num_vertices = nrow(H),
+            direction = FALSE,
+            edge_weights = edge_weights,
+            resolution = resolution,
+            niter = nIterations,
+            nrep = nRandomStarts
         )
     } else {
         edgeOutPath <- tempfile(pattern = "edge_", fileext = ".txt")
@@ -121,22 +125,29 @@ runCluster <- function(
     names(clusts) <- colnames(object)
     rownames(snn) <- colnames(object)
     colnames(snn) <- colnames(object)
-    clusts <- groupSingletons(ids = clusts, SNN = snn,
-                              groupSingletons = groupSingletons,
-                              verbose = verbose)
+    clusts <- groupSingletons(
+        ids = clusts,
+        SNN = snn,
+        groupSingletons = groupSingletons,
+        verbose = verbose
+    )
     cellMeta(object, clusterName, check = FALSE) <- clusts
-    if (isTRUE(saveSNN)) object@uns$snn <- snn
+    if (isTRUE(saveSNN))
+        object@uns$snn <- snn
     if (isTRUE(verbose))
-        cli::cli_process_done(
-            msg_done = "{method} clustering on {type} cell factor loadings ... Found {nlevels(clusts)} cluster{?s}."
-        )
-    object@uns$defaultCluster <- object@uns$defaultCluster %||% clusterName
-    if (isTRUE(verbose))
-        cli::cli_alert_info("{.field cellMeta} variable {.val {clusterName}} is now set as default.")
+        cli::cli_process_done(msg_done = "{method} clustering on {type} cell factor loadings ... Found {nlevels(clusts)} cluster{?s}.")
+    if (is.null(object@uns$defaultCluster)) {
+        # If no default set yet
+        object@uns$defaultCluster <- clusterName
+        if (isTRUE(verbose)) {
+            cli::cli_alert_info("{.field cellMeta} variable {.val {clusterName}} is now set as default.")
+        }
+    }
+
     return(object)
 }
 
-#' [Deprecated] Louvain algorithm for community detection
+#' `r lifecycle::badge("superseded")` Louvain algorithm for community detection
 #' @description
 #' After quantile normalization, users can additionally run the Louvain
 #' algorithm for community detection, which is widely used in single-cell
@@ -172,26 +183,32 @@ NULL
 #' as the replacement, while \code{\link{runCluster}} with default
 #' \code{method = "leiden"} is more recommended.
 #' @export
-louvainCluster <- function( # nocov start
-        object,
-        resolution = 1.0,
-        k = 20,
-        prune = 1 / 15,
-        eps = 0.1,
-        nRandomStarts = 10,
-        nIterations = 100,
-        random.seed = 1,
-        verbose = getOption("ligerVerbose", TRUE),
-        dims.use = NULL
-) {
-    lifecycle::deprecate_warn(
-        "1.99.0", "louvainCluster()",
-        details = "Please use `runCluster()` with `method = 'louvain'` instead.")
+louvainCluster <- function(# nocov start
+    object,
+    resolution = 1.0,
+    k = 20,
+    prune = 1 / 15,
+    eps = 0.1,
+    nRandomStarts = 10,
+    nIterations = 100,
+    random.seed = 1,
+    verbose = getOption("ligerVerbose", TRUE),
+    dims.use = NULL) {
+    lifecycle::deprecate_warn("1.99.0", "louvainCluster()", details = "Please use `runCluster()` with `method = 'louvain'` instead.")
     runCluster(
-        object, method = "louvain", resolution = resolution, nNeighbors = k,
-        prune = prune, eps = eps, nRandomStarts = nRandomStarts,
-        nIterations = nIterations, useDims = dims.use, groupSingletons = TRUE,
-        clusterName = "louvain_cluster", seed = random.seed, verbose = verbose
+        object,
+        method = "louvain",
+        resolution = resolution,
+        nNeighbors = k,
+        prune = prune,
+        eps = eps,
+        nRandomStarts = nRandomStarts,
+        nIterations = nIterations,
+        useDims = dims.use,
+        groupSingletons = TRUE,
+        clusterName = "louvain_cluster",
+        seed = random.seed,
+        verbose = verbose
     )
 } # nocov end
 
@@ -205,12 +222,10 @@ louvainCluster <- function( # nocov start
 # @param verbose Print message
 # @return Returns updated cluster assignment (vector) with all singletons merged
 # with most connected cluster
-groupSingletons <- function(
-        ids,
-        SNN,
-        groupSingletons = TRUE,
-        verbose = FALSE
-) {
+groupSingletons <- function(ids,
+                            SNN,
+                            groupSingletons = TRUE,
+                            verbose = FALSE) {
     # identify singletons
     ids <- droplevels(ids)
     singletons <- names(which(table(ids) == 1))
@@ -259,7 +274,7 @@ groupSingletons <- function(
 .labelClustBySize <- function(clusts) {
     clusts <- as.character(clusts)
     count <- data.frame(table(clusts))
-    count <- count[order(count$Freq, decreasing = TRUE),]
+    count <- count[order(count$Freq, decreasing = TRUE), ]
     map <- seq(0, nrow(count) - 1)
     names(map) <- count[[1]]
     newClusts <- map[clusts]
@@ -287,13 +302,8 @@ groupSingletons <- function(
 #' @examples
 #' pbmc <- mapCellMeta(pbmc, from = "dataset", newTo = "modal",
 #'                     ctrl = "rna", stim = "rna")
-mapCellMeta <- function(
-        object,
-        from,
-        newTo = NULL,
-        ...
-) {
-    object <- recordCommand(object, ...)
+mapCellMeta <- function(object, from, newTo = NULL, ...) {
+    # object <- recordCommand(object, ...)
     from <- cellMeta(object, from)
     if (!is.factor(from))
         cli::cli_abort("{.var from} must be a {.cls factor}.")
@@ -301,17 +311,21 @@ mapCellMeta <- function(
     fromCats <- names(mapping)
     notFound <- fromCats[!fromCats %in% levels(from)]
     if (length(notFound) > 0) {
-        cli::cli_abort("{length(notFound)} categor{?y is/ies are} requested but not found: {.val {notFound}}")
+        cli::cli_abort(
+            "{length(notFound)} categor{?y is/ies are} requested but not found: {.val {notFound}}"
+        )
     }
 
     toCats <- unlist(mapping)
     unchangedCats <- levels(from)
     unchangedCats <- unchangedCats[!unchangedCats %in% fromCats]
     names(unchangedCats) <- unchangedCats
-    if (length(unchangedCats) > 0) toCats <- c(toCats, unchangedCats)
+    if (length(unchangedCats) > 0)
+        toCats <- c(toCats, unchangedCats)
     to <- toCats[as.character(from)]
     to <- factor(unname(to), levels = unique(toCats))
-    if (is.null(newTo)) return(to)
+    if (is.null(newTo))
+        return(to)
     cellMeta(object, newTo) <- to
     return(object)
 }
@@ -347,7 +361,8 @@ mapCellMeta <- function(
 #' calculation. Default \code{NULL} uses all datasets.
 #' @param verbose Logical. Whether to show information of the progress. Default
 #' \code{getOption("ligerVerbose")} or \code{TRUE} if users have not set.
-#' @param classes.compare [Deprecated/Renamed]. Use \code{trueCluster} instead.
+#' @param classes.compare `r lifecycle::badge("superseded")` Use
+#' \code{trueCluster} instead.
 #' @return A numeric scalar, the purity of the clustering result indicated by
 #' \code{useCluster} compared to \code{trueCluster}.
 #' @export
@@ -368,14 +383,12 @@ mapCellMeta <- function(
 #' # evaluated
 #' calcPurity(pbmcPlot, trueCluster = "stim_true_label",
 #'            useCluster = "leiden_cluster", useDatasets = "stim")
-calcPurity <- function(
-        object,
-        trueCluster,
-        useCluster = NULL,
-        useDatasets = NULL,
-        verbose = getOption("ligerVerbose", TRUE),
-        classes.compare = trueCluster
-) {
+calcPurity <- function(object,
+                       trueCluster,
+                       useCluster = NULL,
+                       useDatasets = NULL,
+                       verbose = getOption("ligerVerbose", TRUE),
+                       classes.compare = trueCluster) {
     .deprecateArgs(replace = list(classes.compare = "trueCluster"))
     cellIdx <- rownames(cellMeta(object, useDatasets = useDatasets))
     # Ensure that trueCluster is a factor object, as long as `cellIdx`, named
@@ -385,7 +398,9 @@ calcPurity <- function(
     } else {
         if (length(trueCluster) != length(cellIdx)) {
             if (is.null(names(trueCluster))) {
-                cli::cli_abort("Longer/shorter {.var trueCluster} than cells considered requires {.fn names()} to identify matching.")
+                cli::cli_abort(
+                    "Longer/shorter {.var trueCluster} than cells considered requires {.fn names} to identify matching."
+                )
             }
         } else {
             if (is.null(names(trueCluster))) {
@@ -436,12 +451,14 @@ calcPurity <- function(
 #' calculation. Default \code{NULL} uses all datasets.
 #' @param verbose Logical. Whether to show information of the progress. Default
 #' \code{getOption("ligerVerbose")} or \code{TRUE} if users have not set.
-#' @param classes.compare [Deprecated/Renamed]. Use \code{trueCluster} instead.
+#' @param classes.compare . `r lifecycle::badge("superseded")` Use
+#' \code{trueCluster} instead.
 #' @return A numeric scalar, the ARI of the clustering result indicated by
 #' \code{useCluster} compared to \code{trueCluster}.
 #' @export
 #' @references L. Hubert and P. Arabie (1985) Comparing Partitions, Journal of
 #' the Classification, 2, pp. 193-218.
+#' @returns A numeric scalar of the ARI value
 #' @examples
 #' # Assume the true cluster in `pbmcPlot` is "leiden_cluster"
 #' # generate fake new labeling
@@ -459,24 +476,35 @@ calcPurity <- function(
 #' # evaluated
 #' calcARI(pbmcPlot, trueCluster = "stim_true_label",
 #'         useCluster = "leiden_cluster", useDatasets = "stim")
-calcARI <- function(
-        object,
-        trueCluster,
-        useCluster = NULL,
-        useDatasets = NULL,
-        verbose = getOption("ligerVerbose", TRUE),
-        classes.compare = trueCluster
-) {
+#'
+#' # Comparison of the same labeling should always yield 1.
+#' calcARI(pbmcPlot, trueCluster = "leiden_cluster", useCluster = "leiden_cluster")
+calcARI <- function(object,
+                    trueCluster,
+                    useCluster = NULL,
+                    useDatasets = NULL,
+                    verbose = getOption("ligerVerbose", TRUE),
+                    classes.compare = trueCluster) {
     .deprecateArgs(replace = list(classes.compare = "trueCluster"))
     cellIdx <- rownames(cellMeta(object, useDatasets = useDatasets))
     # Ensure that trueCluster is a factor object, as long as `cellIdx`, named
     # by `cellIdx`
     if (length(trueCluster) == 1) {
-        trueCluster <- cellMeta(object, trueCluster, useDatasets = useDatasets)
+        trueCluster <- .fetchCellMetaVar(
+            object,
+            trueCluster,
+            cellIdx = cellIdx,
+            checkCategorical = TRUE,
+            drop = TRUE,
+            droplevels = TRUE,
+            returnList = FALSE
+        )
     } else {
         if (length(trueCluster) != length(cellIdx)) {
             if (is.null(names(trueCluster))) {
-                cli::cli_abort("Longer/shorter {.var trueCluster} than cells considered requires {.fn names()} to identify matching.")
+                cli::cli_abort(
+                    "Longer/shorter {.var trueCluster} than cells considered requires {.fn names} to identify matching."
+                )
             }
         } else {
             if (is.null(names(trueCluster))) {
@@ -490,7 +518,15 @@ calcARI <- function(
     }
 
     useCluster <- useCluster %||% defaultCluster(object)
-    useCluster <- cellMeta(object, useCluster, useDatasets = useDatasets)
+    useCluster <- .fetchCellMetaVar(
+        object,
+        useCluster,
+        cellIdx = cellIdx,
+        checkCategorical = TRUE,
+        drop = TRUE,
+        droplevels = TRUE,
+        returnList = FALSE
+    )
 
     # Copied from mclust package
     useCluster <- as.vector(useCluster)
@@ -503,10 +539,145 @@ calcARI <- function(
     c <- sum(choose(colSums(tab), 2)) - a
     d <- choose(sum(tab), 2) - a - b - c
     ARI <-
-        (a - (a + b) * (a + c)/(a + b + c + d)) /
-        ((a + b + a + c)/2 - (a + b) * (a + c)/(a + b + c + d))
+        (a - (a + b) * (a + c) / (a + b + c + d)) /
+        ((a + b + a + c) / 2 - (a + b) * (a + c) / (a + b + c + d))
     return(ARI)
 }
 
+#' Calculate Normalized Mutual Information (NMI) by comparing two cluster
+#' labeling variables
+#' @description
+#' This function aims at calculating the Normalized Mutual Information for the
+#' clustering result obtained with LIGER and the external clustering (existing
+#' "true" annotation). NMI ranges from 0 to 1, with a score of 0 indicating no
+#' agreement between clusterings and 1 indicating perfect agreement. The
+#' mathematical definition of NMI is as follows:
+#'
+#' \deqn{
+#' H(X) = -\sum_{x \in X}P(X=x)\log_2 P(X=x)
+#' }
+#' \deqn{
+#' H(X|Y) = -\sum_{y \in Y}P(Y=y)\sum_{x \in X}P(X=x|Y=y)\log_2 P(X=x|Y=y)
+#' }
+#' \deqn{
+#' I(X;Y) = H(X) - H(X|Y)
+#' }
+#' \deqn{
+#' NMI(X;Y) = \frac{I(X;Y)}{\sqrt{H(X)H(Y)}}
+#' }
+#'
+#' Where \eqn{X} is the cluster variable to be evaluated and \eqn{Y} is the true
+#' cluster variable. \eqn{x} and \eqn{y} are the cluster labels in \eqn{X} and
+#' \eqn{Y} respectively. \eqn{H} is the entropy and \eqn{I} is the mutual
+#' information.
+#'
+#' The true clustering annotation must be specified as the base line. We suggest
+#' setting it to the object cellMeta so that it can be easily used for many
+#' other visualization and evaluation functions.
+#'
+#' The NMI can be calculated for only specified datasets, since true annotation
+#' might not be available for all datasets. Evaluation for only one or a few
+#' datasets can be done by specifying \code{useDatasets}. If \code{useDatasets}
+#' is specified, the argument checking for \code{trueCluster} and
+#' \code{useCluster} will be enforced to match the cells in the specified
+#' datasets.
+#' @inheritParams calcARI
+#' @export
+#' @return A numeric scalar of the NMI value
+#' @examples
+#' # Assume the true cluster in `pbmcPlot` is "leiden_cluster"
+#' # generate fake new labeling
+#' fake <- sample(1:7, ncol(pbmcPlot), replace = TRUE)
+#' # Insert into cellMeta
+#' pbmcPlot$new <- factor(fake)
+#' calcNMI(pbmcPlot, trueCluster = "leiden_cluster", useCluster = "new")
+#'
+#' # Now assume we got existing base line annotation only for "stim" dataset
+#' nStim <- ncol(dataset(pbmcPlot, "stim"))
+#' stimTrueLabel <- factor(fake[1:nStim])
+#' # Insert into cellMeta
+#' cellMeta(pbmcPlot, "stim_true_label", useDatasets = "stim") <- stimTrueLabel
+#' # Assume "leiden_cluster" is the clustering result we got and need to be
+#' # evaluated
+#' calcNMI(pbmcPlot, trueCluster = "stim_true_label",
+#'         useCluster = "leiden_cluster", useDatasets = "stim")
+#'
+#' # Comparison of the same labeling should always yield 1.
+#' calcNMI(pbmcPlot, trueCluster = "leiden_cluster", useCluster = "leiden_cluster")
+calcNMI <- function(object,
+                    trueCluster,
+                    useCluster = NULL,
+                    useDatasets = NULL,
+                    verbose = getOption("ligerVerbose", TRUE)) {
+    cellIdx <- rownames(cellMeta(object, useDatasets = useDatasets))
+    # Ensure that trueCluster is a factor object, as long as `cellIdx`, named
+    # by `cellIdx`
+    if (length(trueCluster) == 1) {
+        trueCluster <- .fetchCellMetaVar(
+            object,
+            trueCluster,
+            cellIdx = cellIdx,
+            checkCategorical = TRUE,
+            drop = TRUE,
+            droplevels = TRUE,
+            returnList = FALSE
+        )
+    } else {
+        if (length(trueCluster) != length(cellIdx)) {
+            if (is.null(names(trueCluster))) {
+                cli::cli_abort(
+                    "Longer/shorter {.var trueCluster} than cells considered requires {.fn names} to identify matching."
+                )
+            }
+        } else {
+            if (is.null(names(trueCluster))) {
+                cli::cli_alert_warning(
+                    "Assuming unnamed {.var trueCluster} matches with the cells represented by {.code rownames(cellMeta(object, useDatasets = useDatasets))}."
+                )
+                names(trueCluster) <- cellIdx
+            }
+        }
+        trueCluster <- trueCluster[cellIdx]
+    }
 
+    useCluster <- useCluster %||% defaultCluster(object)
+    useCluster <- .fetchCellMetaVar(
+        object,
+        useCluster,
+        cellIdx = cellIdx,
+        checkCategorical = TRUE,
+        drop = TRUE,
+        droplevels = TRUE,
+        returnList = FALSE
+    )
 
+    # H(X)
+    H_X <- .calcEntropy(useCluster)
+    # H(Y)
+    H_Y <- .calcEntropy(trueCluster)
+
+    overlap <- as.matrix(table(useCluster, trueCluster))
+    # H(X|Y = y), returned a vector for each Y = y
+    H_X_Yy <- apply(overlap, 2, function(x) {
+        probs <- x / sum(x)
+        probs <- probs[probs > 0]
+        - sum(probs * log2(probs))
+    })
+    # P(Y = y), a vector for each Y = y
+    PY <- table(trueCluster) / length(trueCluster)
+    # H(X|Y), a scalar
+    H_X_Y <- sum(PY * H_X_Yy)
+    # I(X;Y)
+    I_X_Y <- H_X - H_X_Y
+    NMI <- I_X_Y / sqrt(H_X * H_Y)
+    return(NMI)
+}
+
+# x - A categorical variable
+.calcEntropy <- function(x) {
+    if (is.factor(x))
+        x <- droplevels(x)
+    count <- table(x)
+    prob <- count / sum(count)
+    - sum(prob * log2(prob))
+}
