@@ -229,9 +229,9 @@ createLigerDataset <- function(
     args <- as.list(environment())
     additional <- list(...)
     # Necessary initialization of slots
-    if (is.null(rawData) && is.null(normData)) {
-        cli::cli_abort("At least one of {.field rawData} or {.field normData} has to be provided.")
-    }
+    # if (is.null(rawData) && is.null(normData)) {
+    #     cli::cli_abort("At least one of {.field rawData} or {.field normData} has to be provided.")
+    # }
     # Look for proper colnames and rownames
     cn <- NULL
     rn <- NULL
@@ -241,18 +241,18 @@ createLigerDataset <- function(
     }
     if (!is.null(rawData)) {
         rn <- rownames(rawData)
-        if (!inherits(rawData, "dgCMatrix"))
+        if (!inherits(rawData, "dgCMatrix") && !inherits(rawData, "DelayedArray"))
             rawData <- methods::as(rawData, "CsparseMatrix")
     }
     if (!is.null(normData)) {
         if (is.null(rn)) rn <- rownames(normData)
-        if (!inherits(normData, "dgCMatrix"))
+        if (!inherits(normData, "dgCMatrix") && !inherits(normData, "DelayedArray"))
             normData <- methods::as(normData, "CsparseMatrix")
     }
     if (!is.null(scaleData)) {
         if (is.null(rn)) rn <- rownames(scaleData)
-        if (!inherits(scaleData, "matrix"))
-            scaleData <- methods::as(scaleData, "matrix")
+        if (!inherits(scaleData, "dgCMatrix")  && !inherits(scaleData, "DelayedArray"))
+            scaleData <- methods::as(scaleData, "CsparseMatrix")
     }
     if (is.null(h5fileInfo)) h5fileInfo <- list()
     if (is.null(featureMeta))
@@ -339,6 +339,14 @@ createH5LigerDataset <- function(
         featureMeta = NULL,
         ...
 ) {
+    replaceFunc <- ifelse(endsWith(h5file, 'h5ad'), 'readH5AD', 'read10XH5')
+    lifecycle::deprecate_warn(
+        when = "2.2.0",
+        what = I('Building liger object with specifying paths to HDF5 files'),
+        with = sprintf('%s()', replaceFunc),
+        details = sprintf("It is now possible to load HDF5 backed data into memory with %s() by default, or featuring the on-disk scalability with %s(inMemory = FALSE)",
+                          replaceFunc, replaceFunc)
+    )
     # Currently commented this check because it fails for unknown reason on
     # Windows platform when subsetting H5 to H5, even if the following part
     # works without passing this check
@@ -1195,5 +1203,343 @@ read10XFiles <- function(
     }
     return(data)
 }
+
+#' Read 10X HDF5 file
+#' @rdname read10XH5
+#' @export
+#' @description
+#' Read count matrix from 10X CellRanger HDF5 file. By default, \code{read10XH5}
+#' load scRNA, scATAC or multimodal data into memory (\code{inMemory = TRUE}).
+#' To use LIGER in delayed mode for handling large datasets, set
+#' \code{inMemory = FALSE} to load the data as a \code{DelayedArray} object. The
+#' delayed mode only supports scRNA data for now.
+#' @param filename Character string, path to the HDF5 file.
+#' @param inMemory Logical, whether to load the data into memory. Default
+#' \code{TRUE}. \code{FALSE} loads the data as a \code{DelayedArray} object.
+#' @param useNames Logical, whether to use gene names as row names. Default
+#' \code{TRUE}. \code{FALSE} uses gene IDs instead.
+#' @param featureMakeUniq Logical, whether to make gene names unique. Default
+#' \code{TRUE}.
+#' @return A sparse matrix when only using older CellRanger output HDF5 file or
+#' when only one genome and one modality is detected. When multiple genomes are
+#' available, will return a list for each genome. When using multimodal data,
+#' each genome will be a list of matrices for each modality. The matrix will be
+#' of dgCMatrix class when in memory, or a TENxMatrix object when in delayed
+#' mode.
+#' @examples
+#' matrix <- read10XH5(
+#'     filename = system.file("extdata/ctrl.h5", package = "rliger"),
+#'     inMemory = TRUE
+#' )
+#' class(matrix) # Should show dgCMatrix
+#' if (requireNamespace("HDF5Array", quietly = TRUE)) {
+#'    matrix <- read10XH5(
+#'       filename = system.file("extdata/ctrl.h5", package = "rliger"),
+#'       inMemory = FALSE
+#'    )
+#'    print(class(matrix)) # Should show TENxMatrix
+#' }
+read10XH5 <- function(
+        filename,
+        inMemory = TRUE,
+        useNames = TRUE,
+        featureMakeUniq = TRUE
+) {
+    if (isTRUE(inMemory)) {
+        read10XH5Mem(filename, useNames, featureMakeUniq)
+    } else {
+        read10XH5Delay(filename, useNames, featureMakeUniq)
+    }
+}
+
+#' @rdname read10XH5
+#' @export
+read10XH5Mem <- function(
+        filename,
+        useNames = TRUE,
+        featureMakeUniq = TRUE
+) {
+    # Adopted from Seurat. with minimum modification to adapt to LIGER coding style
+    if (!is.logical(useNames) || length(useNames) != 1) cli::cli_abort("{.field useNames} must be a logical.")
+    if (!file.exists(filename)) {
+        cli::cli_abort("File not found at {.file {filename}}")
+    }
+    infile <- hdf5r::H5File$new(filename = filename, mode = 'r')
+    genomes <- names(infile)
+    output <- list()
+    if (hdf5r::existsGroup(infile, 'matrix')) {
+        # cellranger version 3
+        featureSlot <- ifelse(useNames, 'features/name', 'features/id')
+    } else {
+        featureSlot <- ifelse(useNames, 'gene_names', 'genes')
+    }
+    for (genome in genomes) {
+        # Modern cellranger output H5 really got genomes as the root group
+        # Older ones just got "matrix" which is the "genome" here
+        counts <- infile[[paste0(genome, '/data')]]
+        indices <- infile[[paste0(genome, '/indices')]]
+        indptr <- infile[[paste0(genome, '/indptr')]]
+        shp <- infile[[paste0(genome, '/shape')]]
+        features <- infile[[paste0(genome, '/', featureSlot)]][]
+        barcodes <- infile[[paste0(genome, '/barcodes')]]
+        mat <- Matrix::sparseMatrix(
+            i = indices[] + 1,
+            p = indptr[],
+            x = as.numeric(counts[]),
+            dims = shp[],
+            repr = "T"
+        )
+        if (isTRUE(featureMakeUniq)) {
+            features <- make.unique(features)
+        }
+        rownames(mat) <- features
+        colnames(mat) <- barcodes[]
+        mat <- methods::as(mat, "CsparseMatrix")
+        # Split v3 multimodal
+        if (infile$exists(paste0(genome, '/features/feature_type'))) {
+            types <- infile[[paste0(genome, '/features/feature_type')]][]
+            typesUniq <- unique(types)
+            if (length(typesUniq) > 1) {
+                cli::cli_alert_info(
+                    "Genome {genome} has multiple modalities, returning a list of matrices for this genome"
+                )
+                mat <- sapply(
+                    X = typesUniq,
+                    FUN = function(x) {
+                        return(mat[which(types == x), ])
+                    },
+                    simplify = FALSE,
+                    USE.NAMES = TRUE
+                )
+            }
+        }
+        output[[genome]] <- mat
+    }
+    infile$close_all()
+    if (length(output) == 1) {
+        return(output[[genome]])
+    } else{
+        return(output)
+    }
+}
+
+#' @rdname read10XH5
+#' @export
+read10XH5Delay <- function(
+        filename,
+        useNames = TRUE,
+        featureMakeUniq = TRUE
+) {
+    if (!requireNamespace("HDF5Array", quitely = TRUE)) {
+        cli::cli_abort(c(
+            x = "Package {.pkg HDF5Array} is required for reading 10X H5 data into DelayedArray.",
+            i = "Please install with {.code BiocManager::install('HDF5Array')}."
+        ))
+    }
+    infile <- hdf5r::H5File$new(filename = filename, mode = 'r')
+    genomes <- names(infile)
+    output <- list()
+    if (hdf5r::existsGroup(infile, 'matrix')) {
+        # cellranger version 3
+        featureSlot <- ifelse(useNames, 'features/name', 'features/id')
+    } else {
+        featureSlot <- ifelse(useNames, 'gene_names', 'genes')
+    }
+    for (genome in genomes) {
+        mat <- HDF5Array::TENxMatrix(filepath = filename, group = genome)
+        features <- infile[[paste0(genome, '/', featureSlot)]][]
+        if (isTRUE(featureMakeUniq)) {
+            features <- make.unique(features)
+        }
+        mat@seed@dimnames[[1]] <- features
+        if (infile$exists(paste0(genome, '/features/feature_type'))) {
+            types <- infile[[paste0(genome, '/features/feature_type')]][]
+            typesUniq <- unique(types)
+            if (length(typesUniq) > 1) {
+                cli::cli_abort(
+                    "LIGER currently does not support multimodal data in DelayedArray."
+                )
+            }
+        }
+        output[[genome]] <- mat
+    }
+    infile$close_all()
+    if (length(output) == 1) {
+        return(output[[genome]])
+    } else{
+        return(output)
+    }
+}
+
+#' Read matrix from H5AD file
+#' @rdname readH5AD
+#' @export
+#' @description
+#' Read raw count matrix from H5AD file. By default, \code{readH5AD} load
+#' specified layer into memory (\code{inMemory = TRUE}). To use LIGER in delayed
+#' mode for handling large datasets, set \code{inMemory = FALSE} to load the
+#' data as a \code{DelayedArray} object. Note that only CSR format is supported
+#' for the matrix.
+#' @details
+#' Currently, the only supported H5AD AnnData encoding versions are as follows:
+#'
+#' \itemize{
+#'  \item{\code{adata.X}, \code{adata.raw.X}, or \code{adata.layers['layer']} -
+#'  csr_matrix 0.1.0}
+#'  \item{\code{adata.obs} and \code{adata.var} - dataframe 0.2.0}
+#'  \item{Categoricals in a data frame - categorical 0.2.0}
+#' }
+#'
+#' If users possess H5AD files encoded with older specification, please either
+#' open an issue on GitHub or use R package 'anndata' to manually extract
+#' information.
+#' @param filename Character string, path to the H5AD file.
+#' @param layer Character string specifying the H5 path of raw count data to be
+#' loaded. Use \code{'X'} for \code{adata.X}, \code{'raw/X'} for
+#' \code{adata.raw.X}, or \code{'layers/layer_name'} for
+#' \code{adata.layers['layer_name']}.
+#' @param inMemory Logical, whether to load the data into memory. Default
+#' \code{TRUE}. \code{FALSE} loads the data as a \code{DelayedArray} object.
+#' @param obs Logical, whether to also load the cell metadata from
+#' \code{adata.obs}. Default \code{FALSE}.
+#' @return When loaded in memory, a sparse matrix of class \code{dgCMatrix} will
+#' be returned. When loaded in delayed mode, a \code{TENxMatrix} object will be
+#' returned. If \code{obs = TRUE}, a list containing the matrix and the cell
+#' metadata will be returned.
+#' @examples
+#' tempH5AD <- tempfile(fileext = '.h5ad')
+#' writeH5AD(pbmc, tempH5AD, overwrite = TRUE)
+#' mat <- readH5AD(tempH5AD, layer = 'X')
+#' delayMat <- readH5AD(tempH5AD, layer = 'X', inMemory = FALSE)
+readH5AD <- function(
+        filename,
+        layer,
+        inMemory = TRUE,
+        obs = FALSE
+) {
+    if (isTRUE(inMemory)) readH5ADMem(filename, layer, obs)
+    else readH5ADDelay(filename, layer, obs)
+}
+
+#' @rdname readH5AD
+#' @export
+readH5ADMem <- function(
+        filename,
+        layer,
+        obs = FALSE
+) {
+    dfile <- hdf5r::H5File$new(filename, mode = 'r')
+    layer <- .H5AD.checkLayer(dfile, layer)
+
+    # Processing the matrix
+    res <- switch(
+        EXPR = hdf5r::h5attr(dfile[[layer]], 'encoding-type'),
+        csr_matrix = .H5AD.readCSR(dfile, layer)
+    )
+    obs_index_col <- hdf5r::h5attr(dfile[['obs']], '_index')
+    obs_names <- dfile[['obs']][[obs_index_col]][]
+    var_index_col <- hdf5r::h5attr(dfile[['var']], '_index')
+    var_names <- dfile[['var']][[var_index_col]][]
+    dimnames(res) <- list(var_names, obs_names)
+
+    if (isTRUE(obs)) {
+        obs <- .H5AD.readObs(dfile)
+        res <- list(
+            matrix = res,
+            obs = obs
+        )
+    }
+
+    dfile$close_all()
+    return(res)
+}
+
+.H5AD.checkLayer <- function(dfile, layer) {
+    layerAvail <- character()
+    if (dfile$exists('X')) layerAvail <- c(layerAvail, 'X')
+    if (dfile$exists('raw')) {
+        if (dfile$exists('raw/X')) {
+            layerAvail <- c(layerAvail, 'raw/X')
+        }
+    }
+    if (dfile$exists('layers')) {
+        layerAvail <- c(layerAvail, names(dfile[['layers']]))
+    }
+    if (missing(layer)) {
+        cli::cli_abort(
+            c(x = '{.field layer} must be specified',
+              i = 'Available options include: {.val {layerAvail}}')
+        )
+    }
+    if (!layer %in% layerAvail) {
+        cli::cli_abort(
+            c(x = 'Cannot identify specified layer {.val {layer}}.',
+              i = 'Valid options include: {.val {layerAvail}}')
+        )
+    }
+    return(layer)
+}
+
+.H5AD.readCSR <- function(dfile, layer) {
+    i <- dfile[[layer]][['indices']][] + 1
+    p <- dfile[[layer]][['indptr']][]
+    x <- dfile[[layer]][['data']][]
+    dims <- rev(hdf5r::h5attr(dfile[[layer]], 'shape'))
+    Matrix::sparseMatrix(i = i, p = p, x = x, dims = dims, repr = 'C')
+}
+
+.H5AD.readObs <- function(dfile) {
+    obs <- dfile[['obs']]
+    obs_index_col <- hdf5r::h5attr(obs, '_index')
+    obs_names <- obs[[obs_index_col]][]
+    df <- data.frame(row.names = obs_names)
+    for (i in seq_along(names(obs))) {
+        cn <- names(obs)[i]
+        if (cn == '_index') next
+        if (inherits(obs[[cn]], 'H5D')) df[[cn]] <- obs[[cn]][]
+        if (inherits(obs[[cn]], 'H5Group')) {
+            categories <- obs[[cn]][['categories']]
+            codes <- obs[[cn]][['codes']]
+            fct <- factor(categories[codes[] + 1], levels = categories[])
+            df[[cn]] <- fct
+        }
+    }
+    return(df)
+}
+
+#' @rdname readH5AD
+#' @export
+readH5ADDelay <- function(
+        filename,
+        layer,
+        obs = FALSE
+) {
+    if (!requireNamespace("HDF5Array", quietly = TRUE)) {
+        cli::cli_abort(c(
+            x = "Package {.pkg HDF5Array} is required for reading 10X H5 data into DelayedArray.",
+            i = "Please install with {.code BiocManager::install('HDF5Array')}."
+        ))
+    }
+    dfile <- hdf5r::H5File$new(filename, mode = 'r')
+    layer <- .H5AD.checkLayer(dfile, layer)
+    obs_index_col <- hdf5r::h5attr(dfile[['obs']], '_index')
+    obs_names <- dfile[['obs']][[obs_index_col]][]
+    var_index_col <- hdf5r::h5attr(dfile[['var']], '_index')
+    var_names <- dfile[['var']][[var_index_col]][]
+
+    res <- HDF5Array::TENxMatrix(filepath = filename, group = layer)
+    res@seed@dimnames <- list(var_names, obs_names)
+
+    if (isTRUE(obs)) {
+        obs <- .H5AD.readObs(dfile)
+        res <- list(
+            matrix = res,
+            obs = obs
+        )
+    }
+    dfile$close_all()
+    return(res)
+}
+
 # nocov end
 
