@@ -447,7 +447,7 @@ cli_or <- function(x) cli::cli_vec(x, list("vec-last" = " or "))
             cli::cli_abort(
                 "Package {.pkg scattermore} is needed for rasterizing the scatter
                 plot. Please install it by command:
-                {.code BiocManager::install('scattermore')}"
+                {.code install.packages('scattermore')}"
             )
         }
     }
@@ -623,14 +623,13 @@ cli_or <- function(x) cli::cli_vec(x, list("vec-last" = " or "))
 }
 
 
-splitRmMiss <- function(x, y, rmMiss = TRUE) {
+splitRmMiss <- function(x, y) {
     y <- factor(y)
     y <- droplevels(y)
     matList <- lapply(levels(y), function(lvl) {
         idx <- y == lvl
         xsub <- x[, idx, drop = FALSE]
-        if (rmMiss) xsub[rowSums(xsub) > 0, , drop = FALSE]
-        else xsub
+        return(xsub)
     })
     names(matList) <- levels(y)
     return(matList)
@@ -646,7 +645,7 @@ searchH <- function(object, useRaw = NULL) {
             if (is.null(Ht)) {
                 cli::cli_abort(
                     "No cell factor loading available.
-                    Please run {.fn runIntegration} and {.fn quantileNorm} first."
+                    Please run {.fn runIntegration} and {.fn alignFactors} first."
                 )
             } else {
                 useRaw <- TRUE
@@ -670,8 +669,8 @@ searchH <- function(object, useRaw = NULL) {
             H <- getMatrix(object, "H.norm")
             if (is.null(H)) {
                 cli::cli_abort(
-                    "Quantile-normalized cell factor loading requested but
-                    not found. Please run {.fn quantileNorm} after
+                    "Aligned cell factor loading requested but
+                    not found. Please run {.fn alignFactors} after
                     {.fn runIntegration}."
                 )
             }
@@ -681,96 +680,26 @@ searchH <- function(object, useRaw = NULL) {
     return(list(H = H, useRaw = useRaw))
 }
 
-# Internal development use only
-# use when you have objects created when this package was named by `rliger2`
-# now we rename the package back to `rliger` and you'll need to have the objects
-# compatible with "new" package name.
-rliger2_to_rliger_namespace <- function(obj, dimredName) {
-    # Slots that contain things of rliger2 namespace
-    datasetList <- list()
-    drList <- list()
-    commandList <- list()
 
-    cm <- obj@cellMeta
-    for (i in dimredName) {
-        drList[[i]] <- cm[[i]]
-        cm[[i]] <- NULL
-    }
-
-    for (i in names(obj@datasets)) {
-        ld <- obj@datasets[[i]]
-        basics <- list(
-            rawData = ld@rawData,
-            normData = ld@normData,
-            scaleData = ld@scaleData,
-            H = ld@H,
-            V = ld@V,
-            A = ld@A,
-            B = ld@B,
-            varUnsharedFeatures = ld@varUnsharedFeatures,
-            scaleUnsharedData = ld@scaleUnsharedData,
-            U = ld@U,
-            h5fileInfo = ld@h5fileInfo,
-            featureMeta = ld@featureMeta,
-            colnames = ld@colnames,
-            rownames = ld@rownames
-        )
-        if (inherits(ld, "ligerATACDataset")) {
-            basics <- c(basics, list(
-                Class = "ligerATACDataset",
-                rawPeak = ld@rawPeak,
-                normPeak = ld@normPeak
-            ))
-        } else if (inherits(ld, "ligerRNADataset")) {
-            basics <- c(basics, list(
-                Class = "ligerRNADataset"
-            ))
-        } else if (inherits(ld, "ligerMethDataset")) {
-            basics <- c(basics, list(
-                Class = "ligerMethDataset"
-            ))
-        } else if (inherits(ld, "ligerSpatialDataset")) {
-            basics <- c(basics, list(
-                Class = "ligerSpatialDataset",
-                coordinate = ld@coordinate
-            ))
-        } else {
-            basics <- c(basics, list(
-                Class = "ligerDataset"
-            ))
+.pivot_longer <- function(
+        data,
+        cols,
+        names_to = "name",
+        values_to = "value"
+) {
+    if (is.numeric(cols) || is.logical(cols)) cols <- colnames(data)[cols]
+    if (!is.character(cols))
+        cli::cli_abort("`cols` should be a character vector.")
+    keeps <- setdiff(colnames(data), cols)
+    blocks <- lapply(cols, function(col) {
+        len <- nrow(data)
+        blockData <- list()
+        blockData[[names_to]] <- rep(col, len)
+        blockData[[values_to]] <- data[[col]]
+        for (keep in keeps) {
+            blockData[[keep]] <- data[[keep]]
         }
-        datasetList[[i]] <- do.call("new", basics)
-    }
-
-    for (i in names(obj@commands)) {
-        cmd <- obj@commands[[i]]
-        commandList[[i]] <- methods::new(
-            "ligerCommand",
-            funcName = cmd@funcName,
-            time = cmd@time,
-            call = cmd@call,
-            parameters = cmd@parameters,
-            objSummary = cmd@objSummary,
-            ligerVersion = cmd@ligerVersion,
-            dependencyVersion = cmd@dependencyVersion
-        )
-    }
-
-    new <- methods::new(
-        "liger",
-        datasets = datasetList,
-        cellMeta = cm,
-        varFeatures = obj@varFeatures,
-        W = obj@W,
-        H.norm = obj@H.norm,
-        uns = obj@uns,
-        commands = commandList,
-        version = obj@version
-    )
-
-    for (i in dimredName) {
-        dimRed(new, i) <- drList[[i]]
-    }
-
-    new
+        as.data.frame(blockData)
+    })
+    do.call(rbind, blocks)
 }
